using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace FilesOfDay {
	class Program {
		static long kilo = 1024;
		static long kiriage = 0;
		static SortedDictionary<DateTime, SizeAndCount> sizes = new SortedDictionary<DateTime, SizeAndCount>();

		static void log( string s ) {
			Console.WriteLine( s );
		}

		static void print( string s ) {
			Console.WriteLine( s );
		}

		static void error( string s ) {
			Console.Error.WriteLine( s );
		}

		static void usage() {
			print( "usage: FilesOfDay [-k] [-m] fBNg" );
			print( "    -k t@CɃLoCg؂グ" );
			print( "    -m t@CɃKoCg؂グ" );
		}

		public static string sizeFold( long size ) {
//			string[] unit = { "Bytes", "KiB", "MiB", "GiB", "TiB" };
			string[] unit = { "Bytes", "KB", "MB", "GB", "TB" };
			int unitIndex = 0;
			double res = (double)size;

			for( unitIndex = 0; unitIndex < unit.Length; unitIndex++ ) {
				if( res < kilo ) {
					break;
				}
				res /= (double)kilo;
			}
			string ress = string.Format( ( ( unitIndex == 0 ) ? "{0:N0}" : "{0:N2}" ), res ) + unit[ unitIndex ];
			//			log( "sizeFold: " + string.Format( "{0:N0}", size ) + "" + ress + "łB" );
			return ress;
		}

		static void countFiles( DirectoryInfo dir ) {
//			log( "fBNgF" + dir.Name );
			try {
				FileInfo[] fi = dir.GetFiles();

				foreach( FileInfo f in fi ) {
					DateTime fd1 = f.LastWriteTime;
					DateTime fd = new DateTime( fd1.Year, fd1.Month, fd1.Day );
					long flen = f.Length;

					if( kiriage > 0 ) {
						flen /= kiriage;
						if( ( f.Length % kiriage ) > 0 ) {
							flen++;
						}
						flen *= kiriage;
					}

					try {
						sizes[ fd ].size += flen;
						sizes[ fd ].count++;
					} catch( System.Collections.Generic.KeyNotFoundException ) {
						sizes.Add( fd, new SizeAndCount( flen, 1 ) );
					}

				}
				foreach( DirectoryInfo d in dir.GetDirectories() ) {
					countFiles( d );
				}
			} catch( System.UnauthorizedAccessException e ) {
				error( "ANZXs\F" + e.Message );
			}
		}

		static void Main( string[] args ) {
			int ac = 0;
			string root;

			for( ac = 0; ac < args.Length; ac++ ) {
				if( args[ ac ].Equals( "-k", StringComparison.CurrentCultureIgnoreCase ) ) {
					kiriage = kilo;
					continue;
				}
				if( args[ ac ].Equals( "-m", StringComparison.CurrentCultureIgnoreCase ) ) {
					kiriage = kilo * kilo;
					continue;
				}
				if( args[ ac ].Equals( "-?", StringComparison.CurrentCultureIgnoreCase ) ) {
					usage();
					return;
				}
				if( args[ ac ].Equals( "/?", StringComparison.CurrentCultureIgnoreCase ) ) {
					usage();
					return;
				}
				break;
			}
			if( ( args.Length - ac ) < 1 ) {
				root = ".";
			} else {
				root = args[ ac ];
			}

			countFiles( new DirectoryInfo( root ) );

			long total = 0;
			long count = 0;
			long min = -1;
			long max = 0;
			DateTime mind = new DateTime();
			DateTime maxd = new DateTime();
			foreach( KeyValuePair<DateTime, SizeAndCount> dv in sizes ) {
				total += dv.Value.size;
				count++;
				if( ( min < 0 ) || ( min > dv.Value.size ) ) {
					min = dv.Value.size;
					mind = dv.Key;
				}
				if( max < dv.Value.size ) {
					max = dv.Value.size;
					maxd = dv.Key;
				}
				print( "" + dv.Key.ToShortDateString() + ": " + sizeFold( dv.Value.size ) + " (" + string.Format( "{0:N0}", dv.Value.count ) + " files)" );
			}
			print( "t@CTCYF" + sizeFold( total ) );
			if( count > 0 ) {
				print( "WvF" + string.Format( "{0:N0}", count ) );
				print( "ρF" + sizeFold( total / count ) );
				print( "ŏF" + sizeFold( min ) + "(" + mind.ToShortDateString() + ")" );
				print( "őF" + sizeFold( max ) + "(" + maxd.ToShortDateString() + ")" );
			}

		}
	}

	public class SizeAndCount {
		public long size;
		public long count;

		public SizeAndCount( long s, long c ) {
			size = s;
			count = c;
		}
	}
}
